%common {}

.upload {
    &__audio {
        margin-top: 50px;
    }

    &__desc {}

    &__text {
        margin-top: 50px;
    }

    &ed {
        margin-top: 50px;

        &__file {
            margin: 25px;
            width: fit-content;
            border: 1px solid #333;
            padding: 5px;
            border-radius: 10px;
        }

        &__delete {
            :hover {
                cursor: pointer;
            }
            color: indianred;
        }
    }

}
