@charset "utf-8";

// Import a Google Font
@import url('https://fonts.googleapis.com/css?family=Material+Icons|Material+Icons+Outlined|Lato:400,700');

// Set your brand colors
$purple: #8A4D76;
$pink: #FA7C91;
$brown: #757763;
$black: #111;
$beige-light: #D0D1CD;
$beige-lighter: #EFF0EB;
$white: #fff;
$blue: #07c;

// Update Bulma's global variables
$family-sans-serif: "Lato", sans-serif;
$grey-dark: $black;
$grey-light: $beige-light;
$primary: $purple;
$link: $blue;
$widescreen-enabled: false;
$fullhd-enabled: false;

// Update some of Bulma's component variables
$body-background-color: $white;
$control-border-width: 2px;
$input-border-color: transparent;
$input-shadow: none;

// Import only what you need from Bulma
@import "./utilities/_all.sass";
@import "./base/_all.sass";
@import "./elements/button.sass";
@import "./elements/container.sass";
@import "./elements/title.sass";
@import "./elements/icon.sass";
@import "./form/_all.sass";
@import "./grid/_all.sass";
@import "./components/navbar.sass";
@import "./components/dropdown.sass";
@import "./layout/hero.sass";
@import "./layout/section.sass";
@import "./pages/index.scss"
